@@ -9,18 +9,31 @@
 
 """
 
+import os
 import sys
+from contextlib import suppress
 from ctypes.util import find_library
 
 from . import constants
 from .ffi import ffi
 
-VERSION = __version__ = '1.6.1'
+VERSION = __version__ = '1.7.0'
 # supported version of cairo, used to be pycairo version too:
 version = '1.17.2'
 version_info = (1, 17, 2)
 
 
+# Python 3.8 no longer searches for DLLs in PATH, so we can add everything in
+# CAIROCFFI_DLL_DIRECTORIES manually. Note that unlike PATH, add_dll_directory
+# has no defined order, so if there are two cairo DLLs in PATH we might get a
+# random one.
+dll_directories = os.getenv('CAIROCFFI_DLL_DIRECTORIES')
+if dll_directories and hasattr(os, 'add_dll_directory'):
+    for path in dll_directories.split(';'):
+        with suppress((OSError, FileNotFoundError)):
+            os.add_dll_directory(path)
+
+
 def dlopen(ffi, library_names, filenames):
     """Try various names for the same library, for different platforms."""
     exceptions = []
@@ -28,7 +41,7 @@ def dlopen(ffi, library_names, filenames):
     for library_name in library_names:
         library_filename = find_library(library_name)
         if library_filename:
-            filenames = (library_filename,) + filenames
+            filenames = (library_filename, *filenames)
         else:
             exceptions.append(
                 'no library called "{}" was found'.format(library_name))
@@ -49,7 +62,7 @@ cairo = dlopen(
     ('libcairo.so.2', 'libcairo.2.dylib', 'libcairo-2.dll'))
 
 
-class _keepref(object):
+class _keepref(object):  # noqa: N801
     """Function wrapper that keeps a reference to another object."""
     def __init__(self, ref, func):
         self.ref = ref
